
/*	Hardware definitions

		Data bus:	LCD D0-7

		IORD	RD_
		PB.2	D/C_
		IOWR	WR_
		PE.4	CS_

*/
#define PORTA_AUX_IO

 //SSD1332 register define

#define rSet_Column_Addr	   	0x15
#define rSet_Row_Addr		   	0x75
#define rSet_Contrast_A			   0x81
#define rSet_Contrast_B			   0x82
#define rSet_Contrast_C			   0x83
#define rMaster_Current_Ctrl  	0x87
#define rSet_Remap_DataFormat	 	0xa0
#define rSet_Start_Line			 	0xa1
#define rSet_Disp_Offset			0xa2
#define rSet_Disp_Mode_Nor			0xa4
#define rSet_Disp_Mode_EnOn		0xa5
#define rSet_Disp_Mode_EnOff		0xa6
#define rSet_Disp_Mode_Inverse	0xa7
#define rSet_Multiplex_Ratio		0xa8
#define rSet_Master_Config			0xad
#define rSet_Disp_Off				0xae
#define rSet_Disp_On					0xaf
#define rSet_PowerSave				0xb0
#define rSet_Phase12					0xb1
#define rDispClk_Div					0xb3
#define rSet_Gray_Table				0xb8
#define rEnLine_Gray_Table			0xb9
#define rVpa_Lv_Setup				0xbb
#define rVpb_Lv_Setup				0xbc
#define rVpc_Lv_Setup	  			0xbd
#define rVcom_Lv_Setup				0xbe
#define rNop							0xe3
#define rDraw_Line		  			0x21
#define rDraw_Rectangle				0x22
#define rCopy				 			0x23
#define rDim_Window					0x24
#define rClr_Window					0x25
#define rFill_Ctrl					0x26

//variable type define
typedef unsigned char	Byte;
typedef	unsigned int	Word;


void MsDelay ( int iDelay )
{
	unsigned long ul0;
	ul0 = MS_TIMER;		// get current timer value
	while ( MS_TIMER < ul0 + (unsigned long) iDelay );
}


/* Low level functions */

#asm root
;@sp+2= data to write
;
Write_Com::
	ld hl,(sp+2)				; get cmd value (LSB)
	ld a,l
	ioe ld (0x8000),a			; and write it
	ret

Write_Data::
	ld hl,(sp+2)			; get data value (LSB)
	ld a,l
	ioe ld (0x8001),a			; and write it
	ret

xdumpblk::

		ld		a,xpc						; save xpc (+2 all @SP)
		push	af
		ld		hl, (SP+6)				; Get mem address (long) in BC:DE
      ld		c,l
      ld		b,h
		ld    hl, (SP+4)
      ex		de,hl

		; Convert BC:DE to a normalized XMEM pointer
		call	LongToXaddr				; DE = logical address, A = xpc
		ld    xpc,a

		ld		hl,(sp+8)				; hl=length

		ld		c,h						; c:b=length
		ld		b,l
		ld		a,l
      ex		de,hl						; HL =address
		or		a
		jr		nz,xcbf_loop
		dec	c

xcbf_loop:
      ld 	a,(hl)					; write 2 pixels
      ioe 	ld	(0x8001),a
      inc 	hl
      ld 	a,(hl)
      ioe	ld	(0x8001),a
      inc 	hl
		djnz	xcbf_loop				;
      xor	a
		dec	c
		jp		p,xcbf_loop

xcbf_done:
		pop	af
		ld		xpc,a
		ret

#endasm


#define Read_Data() RdPortE(0x8001)



void SetColumnAddr(unsigned char ColStart,unsigned char ColEnd)
{
	Write_Com(rSet_Column_Addr);
	Write_Com(ColStart);//A[6:0]
	Write_Com(ColEnd);  //B[6:0]
}

void SetRowAddr(unsigned char RowStart,unsigned char RowEnd)
{
	Write_Com(rSet_Row_Addr);
	Write_Com(RowStart);//A[5:0]
	Write_Com(RowEnd);  //B[5:0]
}

void SetContrast_ColorA(unsigned char Data)
{
	Write_Com(rSet_Contrast_A);
	Write_Com(Data);				//default=0x80
}

void SetContrast_ColorB(unsigned char Data)
{
	Write_Com(rSet_Contrast_B);
	Write_Com(Data);				//default=0x80
}

void SetContrast_ColorC(unsigned char Data)
{
	Write_Com(rSet_Contrast_C);
	Write_Com(Data);				//default=0x80
}

void SetMasterCurrent(unsigned char Data)
{
	Write_Com(rMaster_Current_Ctrl);
	Write_Com(Data);			//default=0x0f
}

void SetRemap_Format(unsigned char Data)
{
	Write_Com(rSet_Remap_DataFormat);
	Write_Com(Data);
}

void SetStartLine(unsigned char Data)
{
	Write_Com(rSet_Start_Line);
	Write_Com(Data);			//default=00
}

void SetDispOffset(unsigned char Data)
{
	Write_Com(rSet_Disp_Offset);
	Write_Com(Data);			//default=00
}

void SetNormalDispMode(void)
{
	Write_Com(rSet_Disp_Mode_Nor);	//default mode(POR)
}

void SetEntireDispOn(void)
{
	Write_Com(rSet_Disp_Mode_EnOn);	//entire display on,GS level=63
}

void SetEntireDispOff(void)
{
	Write_Com(rSet_Disp_Mode_EnOff);//entire display off,all pixels turn off
}

void SetEntireDispInv(void)
{
	Write_Com(rSet_Disp_Mode_Inverse);//inverse display
}

void SetMultiplex(unsigned char Data)
{
	Write_Com(rSet_Multiplex_Ratio);
	Write_Com(Data);			//default=63
}

void SetMaster_Config(unsigned char Data)
{
	Write_Com(rSet_Master_Config);
	Write_Com(Data);	//parameter format:1000 1 a2 a1 a0
}

void SetDispOff(void)
{
	Write_Com(rSet_Disp_Off);			//defalu(POR)
}

void SetDispOn(void)
{
	Write_Com(rSet_Disp_On);
}

void SetPowerSave(void)
{
	Write_Com(rSet_PowerSave);
	Write_Com(0x12);
}

void SetPhase12(unsigned char Data)
{
	Write_Com(rSet_Phase12);	//B3~B0: Phase1
	Write_Com(Data);			//B7~B4: Phase2
}

void SetDispClk_Div(unsigned char Data)
{
	Write_Com(rDispClk_Div);	//B3~B0: Divider
	Write_Com(Data);			//B7~B4: Fosc frequency
}

void SetGrayScale(byte Data0,byte Data1,byte Data2,byte Data3,byte Data4,byte Data5)
{
	Write_Com(rSet_Gray_Table);
	Write_Com(Data0);
	Write_Com(Data1);
	Write_Com(Data2);
	Write_Com(Data3);
	Write_Com(Data4);
	Write_Com(Data5);
}

void SetGrayScale1(byte Data6,byte Data7,byte Data8,byte Data9,byte Data10,byte Data11)
{
	Write_Com(Data6);
	Write_Com(Data7);
	Write_Com(Data8);
	Write_Com(Data9);
	Write_Com(Data10);
	Write_Com(Data11);
}
void SetGrayScale2(byte Data12,byte Data13,byte Data14,byte Data15,byte Data16,byte Data17)
{
	Write_Com(Data12);
	Write_Com(Data13);
	Write_Com(Data14);
	Write_Com(Data15);
	Write_Com(Data16);
	Write_Com(Data17);
}

void SetGrayScale3(byte Data18,byte Data19,byte Data20,byte Data21,byte Data22,byte Data23)
{
	Write_Com(Data18);
	Write_Com(Data19);
	Write_Com(Data20);
	Write_Com(Data21);
	Write_Com(Data22);
	Write_Com(Data23);
}

void SetGrayScale4(byte Data24,byte Data25,byte Data26,byte Data27,byte Data28,byte Data29,byte Data30,byte Data31)
{
	Write_Com(Data24);
	Write_Com(Data25);
	Write_Com(Data26);
	Write_Com(Data27);
	Write_Com(Data28);
	Write_Com(Data29);
	Write_Com(Data30);
	Write_Com(Data31);
}

//void SetEnLine_GrayTable(void)
//{
//	Write_Com(rEnLine_Gray_Table);
//}

void SetVpa_Lv(unsigned char Data)
{
	Write_Com(rVpa_Lv_Setup);
	Write_Com(Data);
}

void SetVpb_Lv(unsigned char Data)
{
	Write_Com(rVpb_Lv_Setup);
	Write_Com(Data);
}

void SetVpc_Lv(unsigned char Data)
{
	Write_Com(rVpc_Lv_Setup);
	Write_Com(Data);
}

void SetVcomh(unsigned char Data)
{
	Write_Com(rVcom_Lv_Setup);
	Write_Com(Data);
}

/* High level functions */

void Draw_Line(byte Col_Start,byte Row_Start,byte Col_End,byte Row_End,byte CC_Line,byte CB_Line,byte CA_Line)
{
	Write_Com(rDraw_Line);
	Write_Com(Col_Start);
	Write_Com(Row_Start);
	Write_Com(Col_End);
	Write_Com(Row_End);
	Write_Com(CC_Line);
	Write_Com(CB_Line);
	Write_Com(CA_Line);
}

void Draw_Rectangle(byte Col_Start,byte Row_Start,byte Col_End,byte Row_End,byte CC_Line,byte CB_Line,byte CA_Line,byte CC_Fill,byte CB_Fill,byte CA_Fill)
{
	Write_Com(rDraw_Rectangle);
	Write_Com(Col_Start);
	Write_Com(Row_Start);
	Write_Com(Col_End);
	Write_Com(Row_End);
	Write_Com(CC_Line);
	Write_Com(CB_Line);
	Write_Com(CA_Line);
	Write_Com(CC_Fill);
	Write_Com(CB_Fill);
	Write_Com(CA_Fill);
}

void Copy(byte Col_Start,byte Row_Start,byte Col_End,byte Row_End,byte Col_New_Start,byte Row_New_Start)
{
	Write_Com(rCopy);
	Write_Com(Col_Start);
	Write_Com(Row_Start);
	Write_Com(Col_End);
	Write_Com(Row_End);
	Write_Com(Col_New_Start);
	Write_Com(Row_New_Start);
}

void DimWindow(byte Col_Start,byte Row_Start,byte Col_End,byte Row_End)
{
	Write_Com(rDim_Window);
	Write_Com(Col_Start);
	Write_Com(Row_Start);
	Write_Com(Col_End);
	Write_Com(Row_End);
}

void ClrWindow(byte Col_Start,byte Row_Start,byte Col_End,byte Row_End)
{
	Write_Com(rClr_Window);
	Write_Com(Col_Start);
	Write_Com(Row_Start);
	Write_Com(Col_End);
	Write_Com(Row_End);
}

void Fill(unsigned char Data)
{
	Write_Com(rFill_Ctrl);
	Write_Com(Data);
}


void SSD_init ()
{

// Use Port E bit 4 for Chip Select with 7 wait-states
#define STROBE			0x10
#define CSREGISTER	IB4CR
#define CSSHADOW		IB4CRShadow
#define CSCONFIG		0x48

	// Initialize Port E bit to be a normal I/O pin
	WrPortI(PEFR, &PEFRShadow, (PEFRShadow|STROBE));

	// Initialize Port E bit to be an output pin
	WrPortI(PEDDR, &PEDDRShadow, (PEDDRShadow|STROBE));

	// Initialize Port E bit to be a chip select.
	WrPortI(CSREGISTER, &CSSHADOW, CSCONFIG);

	// Set Port E bit to be clocked by PCLK/2
	WrPortI(PECR, &PECRShadow, (PECRShadow & ~0xFF));
	MsDelay ( 1000 );				// wait for reset


	SetDispOff();
	SetMultiplex(0x3f);			//set MUX ratio=63
	SetStartLine(0x00);
	SetDispOffset(0x00);
//	SetRemap_Format(0x70);     //16-bit color, remapped
	SetRemap_Format(0x30);     //8-bit color, remapped
	SetMasterCurrent(0x0f);
	SetContrast_ColorA(0xff); 	//40 cd/m^2 @12V=0x15     18cd/m^2 @12V=0x02
	SetContrast_ColorB(0xff); 	//
	SetContrast_ColorC(0xff); 	//40 cd/m^2 @13V=0x14

	SetVpa_Lv(0x3f);
	SetVpb_Lv(0x3f);
	SetVpc_Lv(0x3f);
	SetVcomh(0x7f);

	SetPowerSave();
	SetPhase12(0x24);
	SetDispClk_Div(0xd0);	//F=Fosc*1(D*K*64)
									//Default Fosc=0.97M
	SetMaster_Config(0x8e);	//notice the VCC source
	SetDispOn();
}



#define SSD_home() SetColumnAddr(0,95);SetRowAddr(0,63);


void SSD_fill(unsigned char pattern)
{
int i;
	SSD_home();										// address 0,0
   i=96*64;
	while(i--)
			Write_Data(pattern);				// fill with pattern
}

#define SSD_clear() SSD_fill(0)

#define SSD_plot(x,y,a,b,c) Draw_Line(x,y,x,y,a,b,c)


#ximport "gohan.ssd" gohan
#ximport "dbzgroup.ssd" dbzgroup
#ximport "goku.ssd" goku


void SSD_dump (unsigned long imgdata)
{
unsigned int tocopy,len;

	SSD_home();										// address 0,0
	len=96*64;
   imgdata+=sizeof(long);						// point to data
		while(len) {								// send in 2K blocks
			tocopy=2048;
			if(tocopy>len)
				tocopy=len;
			xdumpblk(imgdata,tocopy);
			imgdata+=(tocopy<<1);
			len-=tocopy;
		}
}

#use "6X8L.lib"
#use "12X16L.lib"

typedef struct {
	unsigned long *font;
	unsigned char	lpc;           // lines per character
	unsigned char	Bpcl;          // bytes per character line (1 or 2)
	unsigned char	bpcl;          // bits per character line
} FontInfo;

const static FontInfo fontinfo[]={
{&Font6x8,8,1,6},
{&Font12x16,16,2,12}
};

void SSD_putchr ( int font, char chr , int color, int bcolor)
{
int i,j,ii,jj;
unsigned long address;
int data,aux;

	i=fontinfo[font].lpc;			// lines per character
	ii=fontinfo[font].Bpcl;			// bytes per character line
   jj=fontinfo[font].bpcl;			// bits per character line
	address=*(fontinfo[font].font)+i*ii*(chr-0x20);
	while(i--){
   	data=xgetint(address);
		aux=(data&0xFF)<<8;			// swap bytes
      data=((data>>8)&0xFF)+aux;
      address+=ii;
      j=jj;
      while(j--){
			if(data&0x8000)
				Write_Data(color);
      	else
         	if(bcolor>=0)				// bcolor =-1 => transparent
					Write_Data(bcolor);
            else
            	Read_Data();			// just increment address counter
         data<<=1;
      }
	}
}

void SSD_printat (int font, unsigned int row, unsigned int col, char *ptr, int color, int bcolor)
{
	do {
   	SetColumnAddr(col,col+fontinfo[font].bpcl-1);	  	// bits per character line
		SetRowAddr(row,row+fontinfo[font].lpc-1);	  			// lines per character
		SSD_putchr (font,*ptr++,color,bcolor);
      col+=fontinfo[font].bpcl;
	} 	while (*ptr);
}

/* MAIN PROGRAM */

main()
{
int i;


	SSD_init();
   Fill(1);									// enable fill
	while(1){
		SSD_home();
		SSD_clear();
		Draw_Rectangle(10,10,70,50,7,29,7,20,4,6);
		SSD_printat(1,20,20,"Si!",103,-1);
		MsDelay ( 3000 );						// wait 3 secs
		SSD_clear();

		for(i=0;i<96;i+=2)
			SSD_plot(i,32,15,1,1); 					// x axis
		for(i=0;i<64;i+=2)
			SSD_plot(48,i,15,1,1); 					// y axis
		for(i=-48;i<48;i++)
			SSD_plot(48+i,32-(int)(32*(sin(i*3.14159/48))),2,2,32); // plot sin(x); 0,0 is at top left
		SSD_printat(0,0,0,"sin(x)",127,10);
		MsDelay ( 3000 );						// wait 3 secs
		SSD_printat(0,56,0,"Cika Electronica",255,-1);
		MsDelay ( 3000 );						// wait 3 secs
		SetRemap_Format(0x70);     //16-bit color, remapped
		SSD_dump(gohan);
		MsDelay ( 5000 );						// wait 5 secs
		SSD_dump(goku);
		MsDelay ( 5000 );						// wait 5 secs
		SSD_dump(dbzgroup);
		MsDelay ( 5000 );						// wait 5 secs
		SetRemap_Format(0x30);     //8-bit color, remapped
	}
}












